﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit.Extensions;
using Xunit;
using Ploeh.Samples.Booking.DomainModel;
using Ploeh.AutoFixture.Xunit;
using Moq;
using Ploeh.AutoFixture;

namespace Ploeh.Samples.Booking.DomainModel.UnitTest
{
    public class CompositeReservationRootFacts
    {
        [Theory, AutoDomainData]
        public void SutIsReservationRoot(CompositeReservationRoot sut)
        {
            Assert.IsAssignableFrom<IReservationRepository>(sut);
        }

        [Theory, AutoDomainData]
        public void RootsIsCorrect([Frozen]IReservationRepository[] roots, CompositeReservationRoot sut)
        {
            Assert.True(roots.SequenceEqual(sut.Roots));
        }

        [Fact]
        public void AddReservationAddsReservationToAllComposedRoots()
        {
            // Fixture setup
            var fixture = new Fixture().Customize(new DomainCustomization());
            var rootMocks = fixture.CreateMany<Mock<IReservationRepository>>().ToList();
            fixture.Inject(rootMocks.Select(m => m.Object).ToArray());

            var reservation = fixture.CreateAnonymous<ReservationAcceptedEvent>();

            var sut = fixture.CreateAnonymous<CompositeReservationRoot>();
            // Exercise system
            sut.AddReservation(reservation);
            // Verify outcome
            rootMocks.ForEach(m =>
                m.Verify(r => 
                    r.AddReservation(reservation)));
            // Teardown
        }
    }
}
