﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using Ploeh.AutoFixture.Xunit;
using Xunit;
using Xunit.Extensions;

namespace Ploeh.Samples.Booking.WebModel.UnitTest
{
    public class MonthViewModelFacts
    {
        [Theory, AutoWebData]
        public void DaysIsInstance(MonthViewModel sut)
        {
            IEnumerable<DayViewModel> result = sut.Days;
            Assert.NotNull(result);
        }

        [Theory, AutoWebData]
        public void DaysIsStable(MonthViewModel sut)
        {
            var expected = sut.Days;
            Assert.Equal(expected, sut.Days);
        }

        [Theory, AutoWebData]
        public void DaysIsCorrectWhenUsingGreedyConstructor([Frozen]DayViewModel[] days, [Greedy]MonthViewModel sut)
        {
            Assert.True(days.SequenceEqual(sut.Days));
        }

        [Theory, AutoWebData]
        public void MonthIsCorrectWhenUsingGreedyConstructor([Frozen]int month, [Greedy]MonthViewModel sut)
        {
            Assert.Equal(month, sut.Month);
        }

        [Theory, AutoWebData]
        public void YearIsCorrectWhenUsingGreedyConstructor([Frozen]int year, [Greedy]MonthViewModel sut)
        {
            Assert.Equal(year, sut.Year);
        }

        [Theory, AutoWebData]
        public void DayCountIsCorrectWhenUsingDateBasedConstructor([Frozen]DateTime date, [Modest]MonthViewModel sut)
        {
            var dayCount = new GregorianCalendar().GetDaysInMonth(date.Year, date.Month);
            Assert.Equal(dayCount, sut.Days.Count());
        }

        [Theory, AutoWebData]
        public void DaysHaveCorrectNumbersWhenUsingDateBasedConstructor([Frozen]DateTime date, [Modest]MonthViewModel sut)
        {
            var dayCount = new GregorianCalendar().GetDaysInMonth(date.Year, date.Month);
            Assert.True(Enumerable.Range(1, dayCount).SequenceEqual(sut.Days.Select(m => m.Day)));
        }

        [Theory, AutoWebData]
        public void MonthIsCorrectWhenUsingDateBasedConstructor([Frozen]DateTime date, [Modest]MonthViewModel sut)
        {
            Assert.Equal(date.Month, sut.Month);
        }

        [Theory, AutoWebData]
        public void YearIsCorrectWhenUsingDateBasedConstructor([Frozen]DateTime date, [Modest]MonthViewModel sut)
        {
            Assert.Equal(date.Year, sut.Year);
        }

        [Theory, AutoWebData]
        public void DisplayNameIsCorrect(MonthViewModel sut)
        {
            var aDayInTheMonth = new DateTime(sut.Year, sut.Month, 1);
            var expected = aDayInTheMonth.ToString("MMMM yyyy");
            Assert.Equal<string>(expected, sut.DisplayName);
        }
    }
}
