﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Castle.MicroKernel.Registration;
using Castle.Windsor;
using Castle.MicroKernel.SubSystems.Configuration;
using Ploeh.Samples.Booking.DomainModel;
using Ploeh.Samples.Booking.Azure;
using Microsoft.WindowsAzure;
using Castle.MicroKernel.Resolvers.SpecializedResolvers;
using Microsoft.WindowsAzure.StorageClient;
using Ploeh.Samples.Booking.WebModel;
using Ploeh.Samples.Booking.Smtp;
using System.Net.Mail;
using System.Net;
using Microsoft.WindowsAzure.ServiceRuntime;

namespace Ploeh.Samples.Booking.WorkerRole
{
    public class BookingWorkerRoleWindsorInstaller : IWindsorInstaller
    {
        #region IWindsorInstaller Members

        public void Install(IWindsorContainer container, IConfigurationStore store)
        {
            //container.Kernel.Resolver.AddSubResolver(new CollectionResolver(container.Kernel));

            //container.Register(AllTypes
            //    .FromAssemblyContaining<MakeNewReservationConsumer>()
            //    .BasedOn(typeof(IMessageConsumer<>))
            //    .WithService.AllInterfaces());
            container.Register(Component
                .For<IMessageConsumer<MakeReservationCommand>>()
                .UsingFactoryMethod(k =>
                {
                    var guard = k.Resolve<CapacityGuard>();
                    var first = new CompositeMessageConsumer<MakeReservationCommand>(guard, k.Resolve<ReservationWriter>(), k.Resolve<AcceptingReservationConsumer>());
                    var second = k.Resolve<RejectingReservationConsumer>();
                    return new ConditionalMessageConsumer<MakeReservationCommand>(guard.HasCapacity, first, second);
                }));
            container.Register(Component
                .For<IMessageConsumer<ReservationAcceptedEvent>>()
                .ImplementedBy<AcceptedReserverationMailer>());
            container.Register(Component
                .For<IMessageConsumer<ReservationRejectedEvent>>()
                .ImplementedBy<RejectedReservationMailer>());
            container.Register(Component
                .For<IMessageConsumer<SoldOutEvent>>()
                .ImplementedBy<MonthViewModelUpdater>());
            container.Register(Component
                .For<ReservationWriter>());
            container.Register(Component
                .For<AcceptingReservationConsumer>());
            container.Register(Component
                .For<RejectingReservationConsumer>());
            container.Register(Component
                .For<IReservationRepository>()
                .ImplementedBy<CompositeReservationRoot>()
                .ServiceOverrides(new { roots = new[] { "azureReservationRoot" /*, "publishingReserverationRoot" */ } }));

            //container.Register(AllTypes
            //    .FromAssemblyContaining<MonthViewModelUpdater>()
            //    .BasedOn(typeof(IMessageConsumer<>))
            //    .WithService.AllInterfaces());

            container.Register(Component
                .For<IReservationRepository>()
                .ImplementedBy<AzureReservationRepository>()
                .Named("azureReservationRoot"));
            container.Register(Component
                .For<ReservationContext>());
            container.Register(Component
                .For<MailContext>());
            container.Register(Component
                .For<IChannel>()
                .ImplementedBy<AzureChannel>());
            container.Register(Component
                .For<CapacityGuard>()
                .ServiceOverrides(new { capacityContainer = "capacityContainer" }));
            container.Register(Component
                .For<IMonthViewWriter>()
                .ImplementedBy<AzureMonthViewStore>()
                .ServiceOverrides(new { viewContainer = "viewContainer" }));

            container.Register(Component
                .For<CloudQueue>()
                .UsingFactoryMethod(k =>
                {
                    var queueName = "messagequeue";
                    var client = k.Resolve<CloudQueueClient>();
                    var queue = client.GetQueueReference(queueName);
                    queue.CreateIfNotExist();
                    return queue;
                }));
            container.Register(Component
                .For<CloudBlobContainer>()
                .UsingFactoryMethod(k =>
                {
                    var c = k.Resolve<CloudBlobClient>().GetContainerReference("capacity");
                    c.CreateIfNotExist();
                    return c;
                })
                .Named("capacityContainer"));
            container.Register(Component
                .For<CloudBlobContainer>()
                .UsingFactoryMethod(k =>
                {
                    var c = k.Resolve<CloudBlobClient>().GetContainerReference("view");
                    c.CreateIfNotExist();
                    return c;
                })
                .Named("viewContainer"));
            container.Register(Component
                .For<CloudQueueClient>()
                .UsingFactoryMethod(k => k.Resolve<CloudStorageAccount>().CreateCloudQueueClient()));
            container.Register(Component
                .For<CloudBlobClient>()
                .UsingFactoryMethod(k => k.Resolve<CloudStorageAccount>().CreateCloudBlobClient()));
            container.Register(Component
                .For<CloudStorageAccount>()
                .UsingFactoryMethod(() => CloudStorageAccount.FromConfigurationSetting("DataConnectionString")));

            container.Register(Component
                .For<IReservationRepository>()
                .ImplementedBy<PublishingReserverationRoot>()
                .Named("publishingReserverationRoot"));

            //container.Register(Component
            //    .For<Action<MailMessage>>()
            //    .UsingFactoryMethod<Action<MailMessage>>(k => k.Resolve<SmtpClient>().Send));
            //container.Register(Component
            //    .For<SmtpClient>()
            //    .UsingFactoryMethod(() =>
            //    {
            //        var smtpServerAddress = RoleEnvironment.GetConfigurationSettingValue("SmtpServerAddress");
            //        var smtpUserName = RoleEnvironment.GetConfigurationSettingValue("SmtpUserName");
            //        var smtpPassword = RoleEnvironment.GetConfigurationSettingValue("SmtpPassword");
            //        var client = new SmtpClient(smtpServerAddress);
            //        client.Credentials = new NetworkCredential(smtpUserName, smtpPassword);
            //        return client;
            //    }));

            container.Register(Component
                .For<Action<MailMessage>>()
                .UsingFactoryMethod<Action<MailMessage>>(k => k.Resolve<TableStorageMailClient>().Send));
            container.Register(Component
                .For<TableStorageMailClient>());

            container.Register(Component
                .For<MailAddress>()
                .UsingFactoryMethod(() =>
                {
                    var senderAddress = RoleEnvironment.GetConfigurationSettingValue("EmailSenderSmtpAddress");
                    var senderDisplayName = RoleEnvironment.GetConfigurationSettingValue("EmailSenderDisplayName");
                    return new MailAddress(senderAddress, senderDisplayName);
                }));
        }

        #endregion
    }
}
